/**
 * グリッドの生成を行なうクラス
 */
define([
  'dijit/form/Button',
  'dijit/form/CheckBox',
  'dijit/form/NumberTextBox',
  'dijit/form/RadioButton',
  'dijit/form/SimpleTextarea',
  'dijit/registry',
  'dijit/TooltipDialog',
  'dijit/popup',
  'wagby/DateTextBox',
  'wagby/Select',
  'wagby/TextBox',
  'wagby/ZipCodeComboBox',
  'dojo/_base/array',
  'dojo/_base/declare',
  'dojo/_base/lang',
  'dojo/aspect',
  'dojo/dom',
  'dojo/dom-attr',
  'dojo/dom-class',
  'dojo/dom-construct',
  'dojo/dom-style',
  'dojo/json',
  'dojo/on',
  'dojo/query',
  'dojo/request',
  'wagby/Grid'
], function(
  Button,
  CheckBox,
  NumberTextBox,
  RadioButton,
  SimpleTextarea,
  registry,
  TooltipDialog,
  popup,
  DateTextBox,
  Select,
  TextBox,
  ZipCodeComboBox,
  array,
  declare,
  lang,
  aspect,
  dom,
  domAttr,
  domClass,
  domConstruct,
  domStyle,
  JSON,
  on,
  query,
  request,
  Grid
) {
  return declare('UpdateListGrid', [Grid], {
    constructor: function(args) {
      this.inherited(arguments);
      this.isUsePagination = true;                         // ページネーションを利用
      this.isUseDeleteCheckbox = args.isUseDeleteCheckbox; // 削除チェックボックス表示フラグ
    },

    getCollectionIdProperty: function() {
      return 'jfcCount';
    },

    getCollectionTarget: function() {
      return this.getCollectionTargetBase() + '/updatelist';
    },

    getActionCommand: function() {
      return {
        action_Search: ''
      };
    },

    setStatusAndDeleteCheckboxToColumns: function() {
      this.setDeleteCheckboxToColumns();
      this.setStatusToColumns();
    },

    setDeleteCheckboxToColumns: function() {
      if (!this.isUseDeleteCheckbox) {
        return;
      }
      this.defaultColumns.unshift({
        id: 'jfcdelete',
        field: 'jfcdelete',
        label: res.LABEL_UPDATELIST_DELETE,
        sortable: false,
        style: {
          label: {
            textAlign: 'center',
            verticalAlign: 'middle',
            width: '35px',
          },
          value: {
            textAlign: 'center',
            verticalAlign: 'middle',
          }
        },
        type: 'jfcdelete',
      });
    },

    setStatusToColumns: function() {
      this.defaultColumns.unshift({
        id: 'jfcStatus',
        field: 'jfcStatus',
        label: res.LABEL_UPDATELIST_STATUS,
        sortable: false,
        renderHeaderCell: function(contentNode) {
        },
        style: {
          label: {
            textAlign: 'center',
            verticalAlign: 'middle',
            width: '50px',
          },
          value: {
            textAlign: 'center',
            verticalAlign: 'middle',
          }
        },
        type: 'status',
      });
    },

    init: function() {
      var that = this;
      try {
        this.createStandby();
        this.createGrid();
        this.setEvent();
        this.getFormWidget();
        this.setConfirmDialog();
        this.setResultDialog();
      } catch(e) {
        console.error(e);
      }
    },

    setEvent: function() {
      var that = this;
      // グリッドの全行表示時に最初の入力項目にフォーカスをセットする処理を設定
      aspect.after(this.grid, 'renderArray', function(rows) {
        that.focusFirstWidgetInGrid(rows);
        return rows;
      });
      // ページリンク押下時にフォームデータを送信する処理を設定
      this.setPageLinkEvent();
      // 画面のボタンの押下時の処理を設定
      this.setButtonClickEvent();
    },

    setButtonClickEvent: function() {
      // 保存ボタン押下時の処理を設定
      this.setExecuteButtonClickEvent();
      // 入力チェックボタン押下時の処理を設定
      this.setRefreshButtonClickEvent();
      // データ0件の場合の新規ボタンを設定
      this.createNewButton();
    },

    setExecuteButtonClickEvent: function() {
      this.btnExecute = registry.byId('btnULExecute');
      if (!this.btnExecute) {
        return;
      }
      this.btnExecute.set('tabIndex', 0);
      var that = this;
      this.btnExecute.onClick = function() {
        if (!that.formWidget.validate()) {
          return;
        }
        that.confirmDialog.show();
      }
    },

    setRefreshButtonClickEvent: function() {
      var btnRefresh = registry.byId('btnRefresh');
      if (!btnRefresh) {
        return;
      }
      var that = this;
      btnRefresh.onClick = function() {
        if (!that.formWidget.validate()) {
          return;
        }
        that.sendValues({
          action: 'action_Refresh',
          method: 'POST',
        });
      }
    },

    createNewButton: function() {
      var actionButtonContainer = this.getActionButtonContainer();
      if (!actionButtonContainer) {
        return;
      }
      var buttonNode = domConstruct.create('button', { id: 'btnNewInsert' + capFirst(this.modelId) }, actionButtonContainer);
      this.createDojoNewButton(buttonNode);
    },

    getActionButtonContainer: function() {
      if (isCssLayout()) {
        return query('div.action_button_area')[0];
      } else {
        var actionTable = query('table.action_table td.action_field')[0];
        if (actionTable) {
          return domConstruct.create('div', { class: 'action_button' }, actionTable);
        }
      }
      return null;
    },

    createDojoNewButton: function(node) {
      this.newButton = new Button({
        type: 'button',
        label: res.LABEL_UPDATELIST_NEW_BUTTON,
        onClick: function() {
          doSubmitForm(this.valueNode.form, 'action_ULInsert_', true);
        }
      }, node);
    },

    setConfirmDialog: function() {
      var that = this;
      createConfirmDialog({
        "title": getAppname(),
        "message": res.CONFIRM_MSG_UPDATELIST_SAVE,
        "actionBarTemplate": "<div class='dijitDialogPaneActionBar' data-dojo-attach-point=\"actionBarNode\">\n\t<button data-dojo-type='dijit/form/Button' id='jfcUpdateListGridConfirmDialogBtnOk' type='submit' data-dojo-attach-point=\"okButton\"></button>\n\t<button data-dojo-type='dijit/form/Button' id='jfcUpdateListGridConfirmDialogBtnCancel' type='button'\n\t\t\tdata-dojo-attach-point=\"cancelButton\" data-dojo-attach-event='click:onCancel'></button>\n</div>\n",
        "onExecute": function() {
          that.sendValues({
            action: 'action_ULExecute',
            method: 'PUT',
            href: 'showList' + capFirst(that.modelId) + '.do'
          });
        },
      }, function(dialog) {
        that.confirmDialog = dialog;
      });
    },

    setResultDialog: function() {
      var that = this;
      createConfirmDialog({
        okButtonLabel: res.LABEL_UPDATELIST_RESULTDIALOG_OK,
        cancelButtonLabel: res.LABEL_UPDATELIST_RESULTDIALOG_CANCEL,
        onCancel: function() {
          if (that.grid.gotoPageOrg) {
            that.grid.gotoPageOrg(that.grid._currentPage);
          } else {
            that.grid.refresh();
          }
          this.hide();
        },
        onShow: function() {
          that.standby.hide();
        },
        actionBarTemplate: "<div class='dijitDialogPaneActionBar' data-dojo-attach-point=\"actionBarNode\">\n\t<button data-dojo-type='dijit/form/Button' id='jfcUlConfirmDialogBtnOk' type='submit' data-dojo-attach-point=\"okButton\"></button>\n\t<button data-dojo-type='dijit/form/Button' id='jfcUlConfirmDialogBtnCancel' type='button'\n\t\t\tdata-dojo-attach-point=\"cancelButton\" data-dojo-attach-event='click:onCancel'></button>\n</div>\n",
        style: "width: 400px",
      }, function(dialog) {
        that.resultDialog = dialog;
      });
    },

    getFormId: function() {
      return 'showList' + this.modelId + '_ulp';
    },

    getFormWidget: function() {
      var formId = this.getFormId();
      this.formWidget = registry.byId(formId);
      if (!this.formWidget) {
        throw res.EXCEPTION_MSG_FORM_MISSING;
      }
    },

    setPageLinkEvent: function() {
      var that = this;
      this.grid.gotoPageOrg = this.grid.gotoPage;
      this.grid.gotoPage = function(page) {
        var action = 'action_Jump_' + ((page - 1) * that.grid.get('rowsPerPage'));
        that.sendValues({
          action: action,
          method: 'POST',
          page: page,
        });
      }
    },

    setColumnSetting: function() {
      this.inherited(arguments);
      if (!this.grid.columns || !this.defaultColumns) {
        return;
      }
      for (var i=0,l=this.defaultColumns.length; i<l; i++) {
        var column = this.defaultColumns[i];
        this.setRenderCellFunc(column.id, column);
      }
    },

    getRenderCellFunc: function(column) {
      var that = this;
      var styles = this.getValueStyles(column);
      return function(object, value, cell, options) {
        if (Object.keys(styles).length > 0) {
          domStyle.set(cell, styles);
        }
        // 入力タイプ無し、更新可能フラグが false の場合はデータ表示のみ
        if (!column.type || column.type !== 'status' && !object.jfceditable) {
          that.grid._defaultRenderCell(object, value, cell, options);
        } else {
          that.setWidget(column, object, value, cell);
        }
      };
    },

    setWidget: function(column, object, value, node) {
      var widget;
      if (column.type === 'text' || column.type === 'email' || column.type === 'url' ||
          column.type === 'status' || column.type === 'hidden') {
        var widgets = [];
        var id = this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f';
        if (column.type === 'status') {
          id = id + this.idProperty;
        } else {
          id = id + column.id;
        }
        var type = 'text';
        if (column.type === 'status' || column.type === 'hidden') {
          type = 'hidden';
        }
        var widget = this.createTextBox({
          id: id,
          type: type,
          style: 'width: 99%;',
          node: node,
        });
        if (column.type === 'status') {
          // ステータス列に主キー値を隠し項目で保持する
          widget.set('value', object[this.idProperty]);
          // ステータスメッセージがある場合、表示する
          if (value) {
            node.appendChild(document.createTextNode(value));
          }
          // エラーがある場合、ツールチップを表示する
          this.setErrorTooltip(object, node, widgets);
        } else {
          widget.set('_onChangeActive', false);
          widget.set('value', value);
          widget.set('_onChangeActive', true);
          this.setRefreshFuncOnChange(column, widget);
        }
        widgets.push(widget);
        node.widgets = widgets;
      } else if (column.type === 'check' || column.type === 'pseudo_checkbox') {
        var widgets = [];
        var values = [];
        if (value) {
          values = value.split(',');
        }
        var selectionData = this.getSelectionData(object[column.field]);
        var options = selectionData.options;
        for (var i=0,l=options.length; i<l; i++) {
          var optionValue = options[i].value;
          if (column.pseudoCheckValue && column.pseudoCheckValue != optionValue) {
            continue;
          }
          var container = domConstruct.create('div', { style: 'white-space: nowrap; float: left;' }, node);
          var optionLabel = options[i].label;
          var id = this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id + '_005b' + optionValue + '_005d_002f_0040id';
          var widget = new CheckBox({
            id: id,
            name: id,
            value: optionValue
          }).placeAt(container);
          if (selectionData.selectedValues[optionValue]) {
            widget.set('checked', true);
          }
          this.setRefreshFuncOnChange(column, widget);
          domConstruct.create('label', { innerHTML: optionLabel, for: id, style: 'padding-left: 2px; padding-right: 10px;' }, container);
          widgets.push(widget);
        }
        node.widgets = widgets;
      } else if (column.type === 'number') {
        widget = this.getNumberTextBox({
          id: this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id,
          value: value,
          style: 'width: 99%;',
          format: column.format,
        });
        widget.placeAt(node);
        this.setRefreshFuncOnChange(column, widget);
        node.widgets = [widget];
      } else if (column.type === 'dateTextbox') {
        widget = this.getDateTextBox({
          id: this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id,
          value: value,
          style: 'width: 95%',
          format: column.format,
        });
        widget.placeAt(node);
        this.setRefreshFuncOnChange(column, widget);
        node.widgets = [widget];
      } else if (column.type === 'radio') {
        this.createRadioButtons({
          column: column,
          node: node,
          object: object,
          value: value,
        });
      } else if (column.type === 'combo') {
        var selectionData = this.getSelectionData(object[column.field]);
        var selectedValue = " ";
        for (var key in selectionData.selectedValues) {
          selectedValue = key;
        }
        var id = this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id + '_005b1_005d_002f_0040id';
        var widget = new Select({
          id: id,
          name: id,
          value: selectedValue,
          options: selectionData.options,
        }).placeAt(node);
        this.setRefreshFuncOnChange(column, widget);
        node.widgets = [widget];
      } else if (column.type === 'search') {
        this.createSearch({
          column: column,
          node: node,
          object: object,
          value: value,
        });
      } else if (column.type === 'textarea') {
        var widget = this.createSimpleTextarea({
          id: this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id,
          column: column,
          node: node,
          object: object,
          value: value,
        });
        node.widgets = [widget];
        this.setRefreshFuncOnChange(column, widget);
      } else if (column.type === 'barGauge') {
        var props = this.getBarGaugeProps(column);
        var that = this;
        setTimeout(function() {
          // バーゲージを配置するDOMをグリッドの行表示後に生成しないと
          // 高さ、幅が取得できないため、表示を 20ms 遅らせる
          var widget = that.getBarGaugeWidget({
            id: that.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id,
            value: value,
            props: props,
            node: node,
          });
          that.setRefreshFuncOnChange(column, widget);
          node.widgets = [widget];
        }, 20);
      } else if (column.type === 'zipcode') {
        var widget = this.createZipCodeComboBox({
          id: this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.id,
          target: this.modelId + '_ulp_002fitem_005b' + object.jfcCount + '_005d_002f' + column.addressItemId,
          node: node,
          object: object,
          value: value,
        });
        this.setRefreshFuncOnChange(column, widget);
        node.widgets = [widget];
      } else if (column.type === 'jfcdelete') {
        if (!object.jfcdeleteable) {
          return;
        }
        var id = 'jfcdelete_' + this.modelId + '_' + (object.jfcCount - 1);
        var widget = new CheckBox({
          id: id,
          name: id,
          tabIndex: -1,
        }).placeAt(node);
        on(node, 'click', function(e) {
          if (e.type === 'click' && e.target.nodeName === 'TD') {
            widget.set('checked', !widget.get('checked'));
          }
        });
        node.widgets = [widget];
      }
    },

    setRefreshFuncOnChange: function(column, widget) {
      if (!column.refreshOnChangeFunc) {
        return;
      }
      widget.on("change", function() {
        column.refreshOnChangeFunc();
      });
    },

    createTextBox: function(args) {
      var widget = new TextBox({
        id: args.id,
        name: args.id,
        type: args.type,
        value: args.value || '',
      });
      if (args.readOnly) {
        widget.set('readOnly', args.readOnly);
      }
      if (args.style) {
        widget.set('style', args.style);
      }
      widget.placeAt(args.node);
      return widget;
    },

    getNumberTextBox: function(args) {
      var widget = new NumberTextBox({
        id: args.id,
        name: args.id,
      });
      if (args.style) {
        widget.set('style', args.style);
      }
      var value = args.value || '';
      if (args.format) {
        widget.set('constraints', {
          pattern: args.format
        });
        value = widget._formatter(value, widget.get('constraints'));
      }
      widget.set('_onChangeActive', false);
      widget.set('value', value);
      widget.set('_onChangeActive', true);
      return widget;
    },

    getDateTextBox: function(args) {
      var widget = new DateTextBox({
        id: args.id,
        name: args.id,
      });
      if (args.style) {
        widget.set('style', args.style);
      }
      if (args.format) {
        widget.set('dateFormat', args.format);
      }
      widget.set('_onChangeActive', false);
      widget.set('value', args.value || '');
      widget.set('_onChangeActive', true);
      return widget;
    },

    createSimpleTextarea: function(args) {
      var widget = new SimpleTextarea({
        id: args.id,
        name: args.id,
        value: args.value || '',
        style: 'padding-right: 0; width: 98%',
      }).placeAt(args.node);
      if (args.style) {
        widget.set('style', args.style);
      }
      return widget;
    },

    createZipCodeComboBox: function(args) {
      var widget = new ZipCodeComboBox({
        id: args.id,
        name: args.id,
        value: args.value || '',
        target: args.target,
      }).placeAt(args.node);
      if (args.style) {
        widget.set('style', args.style);
      }
      return widget;
    },

    createButton: function(args) {
      var button = this.inherited(arguments);
      button.set('tabIndex', '-1');
      domStyle.set(button.domNode, 'margin', '0 1px 0 0');
      domStyle.set(button.domNode.firstChild, 'padding', '0 3px');
      if (args.style) {
        button.set('style', args.style);
      }
      if (args.title) {
        button.set('title', args.title);
      }
      return button;
    },

    createRadioButtons: function(args) {
      var widgets = [];
      var container = domConstruct.create('div', { style: 'float: left; margin: 0 3px;' }, args.node);
      widgets.push(this.createButton({
        id: this.modelId + '_ulp_' + args.column.id + 'updateList_clear' + args.object.jfcCount,
        node: container,
        iconClass: 'clearButtonIcon',
        title: res.LABEL_UPDATELIST_CLEAR_BUTTON,
        onClick: function() {
          for (var i=0,l=widgets.length; i<l; i++) {
            widgets[i].set('checked', false);
          }
        }
      }));
      var selectionData = this.getSelectionData(args.object[args.column.field]);
      var options = selectionData.options;
      for (var i=0,l=options.length; i<l; i++) {
        var optionValue = options[i].value;
        var optionLabel = options[i].label;
        var container = domConstruct.create('div', { style: 'white-space: nowrap; float: left;' }, args.node);
        var id = this.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b' + optionValue + '_005d_002f_0040id';
        var name = this.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id';
        var widget = new RadioButton({
          id: id,
          name: name,
          value: optionValue,
        }).placeAt(container);
        if (selectionData.selectedValues[optionValue]) {
          widget.set('checked', true);
        }
        this.setRefreshFuncOnChange(args.column, widget);
        domConstruct.create('label', { innerHTML: optionLabel, for: id, style: 'padding-left: 2px; padding-right: 5px;' }, container);
        widgets.push(widget);
      }
      args.node.widgets = widgets;
    },

    createSearch: function(args) {
      args.node.widgets = [];
      var values = {
        id: '',
        content: ''
      };
      var fieldValues = args.object[args.column.field];
      if (fieldValues && fieldValues.length > 0) {
        values.id = fieldValues[0].id;
        values.content = fieldValues[0].content;
      }
      var container = domConstruct.create('div', { style: 'white-space: nowrap; float: left;' }, args.node);
      args.node.widgets.push(this.createButton({
        id: this.modelId + '_ulp_' + args.column.id + 'updateList_clear' + args.object.jfcCount,
        node: container,
        iconClass: 'clearButtonIcon',
        title: res.LABEL_UPDATELIST_CLEAR_BUTTON,
        onClick: function() {
          args.node.widgets[1].set('value', '');
          args.node.widgets[2].set('value', '');
        }
      }));
      var widget = this.createTextBox({
        id: this.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id',
        type: 'hidden',
        value: values.id,
        node: container,
      });
      args.node.widgets.push(widget);
      this.setRefreshFuncOnChange(args.column, widget);
      args.node.widgets.push(this.createTextBox({
        id: this.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id_content',
        type: 'text',
        value: values.content,
        style: 'width: 55%; margin: 0 5px;',
        readOnly: true,
        node: container,
      }));

      var that = this;
      args.node.widgets.push(this.createButton({
        id: 'btn_' + this.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id',
        node: container,
        iconClass: "dijitIconSearch",
        title: res.LABEL_UPDATELIST_SEARCH_BUTTON,
        onClick: function() {
          if (!args.column.searchWindow) {
            return;
          }
          openSearchWindow(
            args.column.searchWindow.actionName,
            'item_' + args.column.id,
            that.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id',
            that.modelId + '_ulp_002fitem_005b' + args.object.jfcCount + '_005d_002f' + args.column.id + '_005b1_005d_002f_0040id_content',
            args.column.searchWindow.contentElement,
            args.column.searchWindow.size
          );
        },
      }));
    },

    setErrorTooltip: function(object, node, widgets) {
      if (!object.jfcerror || object.jfcerror.length === 0) {
        return;
      }
      var errorMessage = "";
      for (var i=0,l=object.jfcerror.length; i<l; i++) {
        errorMessage = errorMessage + object.jfcerror[i] + '<br>';
      }
      var tooltipDialog = new TooltipDialog({
        id: 'jfcErrorTooltip' + object.jfcCount,
        content: errorMessage,
        onMouseLeave: function() {
          popup.close(tooltipDialog);
        }
      });
      domConstruct.create('span', {
        id: 'jfcErrorMessageContainer' + object.jfcCount,
        innerHTML: res.ERROR_MSG_UPDATELIST_ERROR,
        style: 'color: #990000',
        onmouseover: function(e) {
          popup.open({
            popup: tooltipDialog,
            around: this.id
          });
        }
      }, node);
      widgets.push(tooltipDialog);
    },

    setGridSelectionMode: function(isLinkDefined) {
      this.grid.set('selectionMode', 'none');
    },

    parseJson: function(data) {
      return JSON.parse(data, reviver);

      function reviver(key, value) {
        if (key === "get" && typeof value === "string" && value.match(/^function/)) {
          return Function.call(this, "return " + value)();
        }
        return value;
      }
    },

    getParseFunc: function() {
      var that = this;
      return function(response) {
        var responseData = that.parseJson(response);
//console.log('responseData: ', responseData);
        // 検索結果から現在表示されているページをクッキーへ保存する
        that.saveCurrentPageSettings(responseData);
        var total = responseData.total;
        var items = [];
        var listulpItems = responseData.listulp.item;
        for (var i=0,l=listulpItems.length; i<l; i++) {
          var item = responseData.entity[i];
          var listulpItem = listulpItems[i];
          for (var key in listulpItem) {
            if (!listulpItem[key]) {
              continue;
            }
            if (listulpItem[key] instanceof Array) {
              item[key] = listulpItem[key];
            } else if (key.lastIndexOf("jfcButtonField", 0) === 0) {
              item[key] = listulpItem[key];
            } else {
              if (listulpItem[key].content) {
                item[key] = listulpItem[key].content;
              }
            }
          }
          item.jfcCount = i + 1;
          item.jfcStatus = that.getJfcStatusMsg(listulpItem);
          item.jfcdeleteable = listulpItem.jfcdeleteable;
          item.jfceditable = listulpItem.jfceditable;
          item.jfcedited = listulpItem.jfcedited;
          item.jfcinsertable = listulpItem.jfcinsertable;
          var pkey;
          // 複合主キーの場合、連結した値をセットする
          if (that.idProperty.indexOf(that.CompositePrimaryKeySeparator) !== -1) {
            item[that.idProperty] = that.getPrimaryKeyValue(responseData.entity[i]);
            pkey = item[that.idProperty];
          } else {
            pkey = listulpItem[that.idProperty].content;
          }
          items.push(item);
          if (that.isNewData(listulpItem)) {
            total++;
          }
        }
        that.postProcessAfterGetResponse(responseData);
        that.showNewButton(total);
//console.log('items: ', items);
//console.log('total: ', total);
        return {
          items: items,
          total: total
        }
      }
    },

    postProcessAfterGetResponse: function(responseData) {
      if (!this.defaultColumns) {
        this.defaultColumns = lang.clone(responseData.gridColumns);
        // columns に削除用の行選択チェックボックス設定を追加する
        this.setStatusAndDeleteCheckboxToColumns();
      }
      this.setColumns();
      // 主キー値をセットする
      this.setPrimaryKeyData(responseData);
    },

    setPrimaryKeyData: function(responseData) {
      for (var i=0,l=responseData.entity.length; i<l; i++) {
        var entity = responseData.entity[i];
        entity[this.ID_VALUE_KEY] = this.getPrimaryKeyValue(entity);
      }
    },

    isNewData: function(listulpItem) {
      if (listulpItem.jfcupdatetype === "insert" ||
          listulpItem.jfcupdatetype === "copy") {
        return true;
      }
      return false;
    },

    showNewButton: function(total) {
      if (total > 0) {
        this.newButton.set('style', 'display: none');
      } else {
        this.newButton.set('style', 'display: inline');
      }
    },

    focusFirstWidgetInGrid: function(rows) {
      var that = this;
      setTimeout(function() {
        var allWidgets = registry.findWidgets(that.grid.contentNode);
        if (!allWidgets || allWidgets.length === 0) {
          return;
        }
        // 最終行でフォーカスを外した場合は保存ボタンへフォーカスがセットされるようにする
        that.setBlurEventLastWidget(allWidgets);
        var focusWidget = that.getFocusWidget(allWidgets);
        if (!focusWidget) {
          return;
        }
        // 新規、コピー実行時は行内のウィジェットにフォーカスをセットする
        if (that.previousAction) {
          var index = that.previousAction.jfcCount;
          if (that.previousAction.actionName.lastIndexOf('action_Insert') === 0 &&
              that.previousAction.jfcCount === 1) {
            // 1行目で新規登録を行なった場合は、1行目に追加される
            index = 0;
          }
          var widget = that.getFocusWidget(registry.findWidgets(rows[index]));
          if (widget) {
            focusWidget = widget;
          }
        }
        if (focusWidget) {
          focusWidget.focus();
        }
      }, 1);
    },

    setBlurEventLastWidget: function(widgets) {
      if (!this.btnExecute) {
        return;
      }
      var that = this;
      for (var i=widgets.length - 1; i>=0; i--) {
        var widget = widgets[i];
        if (!this.isFocusableWidget(widget)) {
          continue;
        }
        widget.on('blur', function() {
          that.btnExecute.focus();
        });
        break;
      }
    },

    getFocusWidget: function(widgets) {
      for (var i=0,l=widgets.length; i<l; i++) {
        var widget = widgets[i];
        if (!this.isFocusableWidget(widget)) {
          continue;
        }
        return widget;
      }
      return null;
    },

    isFocusableWidget: function(widget) {
      if (widget.type == "hidden" || widget.type == "button" || widget.get("disabled") ||
          widget.get("readOnly") || widget.get("tabIndex") === -1) {
        return false;
      }
      return true;
    },

    getJfcStatusMsg: function(listulpItem) {
      if (!listulpItem.jfclocked) {
        return res.ERROR_MSG_UPDATELIST_LOCKERROR;
      } else if (listulpItem.jfcedited) {
        return res.INFO_MSG_UPDATELIST_UPDATE;
      } else if (listulpItem.jfcupdatetype === "insert") {
        return res.INFO_MSG_UPDATELIST_NEW;
      } else if (listulpItem.jfcupdatetype === "copy") {
        return res.INFO_MSG_UPDATELIST_COPY;
      }
    },

    getSelectionData: function(entityData) {
      var selectionData = {
        'selectedValues': {},
        'options': [],
      };
      if (!entityData || entityData.length === 0) {
        return selectionData;
      }
      for (var i=0,l=entityData.length; i<l; i++) {
        var value = entityData[i].id + "";
        if (entityData[i].noId) {
          value = " ";
        }
        if (entityData[i].choose) {
          selectionData.selectedValues[value] = true;
        }
        selectionData.options.push({
          label: entityData[i].content,
          value: value,
        });
      }
      return selectionData;
    },

    getUrl: function(args) {
      this.previousAction = "";
      var url = this.getCollectionTarget();
      if (!args) {
        return url;
      }
      if (args.action.lastIndexOf('action_Jump') === 0) {
        url = url + '/jump/' + (args.page - 1) * this.grid.get('rowsPerPage');
        return url;
      } else if (args.action.lastIndexOf('ULExecute') !== -1) {
        return url;
      } else if (args.action.lastIndexOf('Refresh') !== -1) {
        url = url + '/refresh';
        return url;
      } else if (args.actionName) {
        if (args.actionName.lastIndexOf('action_Copy') === 0) {
          url = url + '/copy/';
          args.action = 'action_ULCopy_';
          args.method = 'GET';
          args.page = null;
        } else if (args.actionName.lastIndexOf('action_Insert') === 0) {
          url = url + '/insert/';
          args.action = 'action_ULInsert_';
          args.method = 'GET';
          args.page = null;
        }
        this.previousAction = args;
      }
      var index = args.jfcCount - 1;
      args.action = args.action + index;
      return url + index;
    },

    getButtonClickEventFunc: function(args) {
      var that = this;
      return function(e) {
        that.sendRequestAfterRefresh(args);
      }
    },

    sendRequestAfterRefresh: function(args) {
      var that = this;
      this.sendValues({
        action: 'action_Refresh',
        method: 'POST',
        callBack: function() {
          that.sendValues(args);
        }
      });
    },

    getButtonId: function(button, object) {
      return 'btn' + button.action + this.modelId + '_' + object.jfcCount;
    },

    getWidgetsInForm: function() {
      var form = this.getForm();
      if (!form) {
        return [];
      }
      return registry.findWidgets(form);
    },

    getWidgetsExceptButtons: function() {
      var widgetsExceptButtons = array.filter(this.getWidgetsInForm(), function(widget) {
        // ボタン以外の入力フォームを返す
        return widget.type !== 'button';
      });
      return widgetsExceptButtons;
    },

    getWagbyFormValues: function() {
      var parameters = {};
      this.setParameters(this.getWidgetsExceptButtons(), parameters);
      for (var key in parameters) {
        var value = parameters[key];
        if (value instanceof Array) {
          // チェックボックスの値は配列では送信しない
          parameters[key] = value[0] || "";
        } else if (value === " ") {
          // 項目のデータベースの型が数値の場合に1文字空白を送信すると
          // エラーが発生するため、空文字に変更する
          parameters[key] = "";
        }
      }
      return parameters;
    },

    sendValues: function(args) {
      var url = this.getUrl(args);
      var requestOptions = {};
      requestOptions.headers = {
        'X-Wagby-RESTAPIVersion': 'v2',
        'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'
      },
      requestOptions.data = this.getWagbyFormValues();
      requestOptions.data[args.action] = "";
      requestOptions.method = args.method;
console.log('requestOptions: ', requestOptions);
      var that = this;
      this.standby.show();
      request(url, requestOptions).then(function(data) {
        if (args.callBack) {
          args.callBack();
        } else if (args.href) {
          var message = that.getResultMessage(data);
          that.resultDialog.set('content', message);
          that.resultDialog.set('onExecute', function() {
            window.location.href = args.href;
          });
          that.resultDialog.show();
        } else if (args.page) {
          if (that.grid.gotoPageOrg) {
            that.grid.gotoPageOrg(args.page);
          } else {
            that.grid.refresh();
          }
        } else {
          if (that.grid.gotoPageOrg) {
            that.grid.gotoPageOrg(that.grid._currentPage);
          } else {
            that.grid.refresh();
          }
        }
        that.standby.hide();
      }, function(err) {
console.error("sendRequest Error: ", err);
        that.standby.hide();
        if (err.response.status === 404) {
          showAlertDialog(res.ERROR_MSG_UPDATELIST_REST);
        } else {
          var message = that.getResultMessage(err.response.data);
          that.resultDialog.set('content', message);
          that.resultDialog.set('onExecute', function() {
            window.location.href = args.href;
          });
          that.resultDialog.show();
        }
      });
    },

    getResultMessage: function(data) {
      var result = JSON.parse(data);
      if (!result.errors) {
        return "";
      }
      var msg = "";
      var jfcerror = result.errors.jfcerror;
      if (jfcerror) {
        for (var i=0,l=jfcerror.length; i<l; i++) {
          msg = msg + '<div style="color: #990000;">' + jfcerror[i].content + '</div>';
        }
      }
      var jfcwarn = result.errors.jfcwarn;
      if (jfcwarn) {
        for (var i=0,l=jfcwarn.length; i<l; i++) {
          msg = msg + '<div style="color: #990000;">' + jfcwarn[i].content + '</div>';
        }
      }
      var jfcinfo = result.errors.jfcinfo;
      if (jfcinfo) {
        for (var i=0,l=jfcinfo.length; i<l; i++) {
          msg = msg + '<div>' + jfcinfo[i].content + '</div>';
        }
      }
      return msg;
    },

    getCookieModelId: function() {
      return 'UpdateList' + capFirst(this.modelId);
    },
  });
});
